/*
 |  xor.c -- XOR file data with data from random/other source
 |  Copyright (c) 2000 by proton. All rights reserved.
 |
 |  This program is free software; you can redistribute it and/or modify
 |  it under the terms of the GNU General Public License as published by
 |  the Free Software Foundation; either version 2 of the License, or
 |  (at your option) any later version.
 |
 |  This program is distributed in the hope that it will be useful,
 |  but WITHOUT ANY WARRANTY; without even the implied warranty of
 |  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 |  GNU General Public License for more details.
 */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>

#undef BUFSIZE
#define BUFSIZE		512

char	membuf[BUFSIZE];
char	xorbuf[BUFSIZE];

char	*progname;

char	*srcfile;
char	*destfile;
char	*maskfile;

struct
{
	char	*opt;
	char	**fileptr;
} option[3] =
{
{	"-i",	&srcfile	},
{	"-o",	&destfile	},
{	"-x",	&maskfile	}
};

void usage(char *err, int perr)
{
	if (err)
		fprintf(stderr,(perr) ? "%s: %s: " : "%s: %s\n",progname,err);
	if (perr)
		perror("");
	fprintf(stderr,"Usage: %s -i<input> -o<output> [-x<xordata>]\n",progname);
	exit(1);
}

int main(int argc, char **argv)
{
	int	i,j,n,o,x;

	progname = argv[0];

	srcfile = destfile = maskfile = NULL;
	i = 1;
nextopt:
	while((i < argc) && (argv[i]))
	{
		for(j=0;j<3;j++)
		{
			if (!strncmp(option[j].opt,argv[i],2))
			{
				if (!argv[i][2])
				{
					i++;
					if ((i >= argc) || (!argv[i]))
						usage("missing arguments",0);
					*option[j].fileptr = argv[i];
				}
				else
				{
					*option[j].fileptr = &argv[i][2];
				}
				i++;
				goto nextopt;
			}
		}
		usage("unknown option",0);
	}
	if (!srcfile)
		usage("no input file",0);
	if ((i = open(srcfile,O_RDONLY)) < 0)
	{
		usage("unable to open input file",1);
	}
	if (!destfile)
		usage("no output file",0);
	if ((o = open(destfile,O_WRONLY|O_TRUNC|O_CREAT,0644)) < 0)
	{
		usage("unable to open output file",1);
	}
	if (!maskfile)
	{
		fprintf(stderr,"%s: no xor data file - reading from stdin\n",progname);
		x = 0;
	}
	else
	if ((x = open(maskfile,O_RDONLY)) < 0)
	{
		usage("unable to open output file",1);
	}

	/*
	 *  ready, set, go!
	 */

	for(;;)
	{
		if ((n = read(i,membuf,sizeof(membuf))) < 0)
		{
			fprintf(stderr,"%s: read error on input file: ",progname);
			perror("");
			exit(1);
		}
		if (!n)
			break;
		if (read(x,xorbuf,n) != n)
		{
			fprintf(stderr,"%s: insufficient data read from XOR source\n",progname);
			exit(1);
		}
		for(j=0;j<n;j++)
			membuf[j] ^= xorbuf[j];
		if ((write(o,membuf,n)) < 0)
		{
			fprintf(stderr,"%s: write error on output file: ",progname);
			perror("");
			exit(1);
		}
	}

	exit(0);
}
